<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('products', function (Blueprint $table) {
            $table->mediumIncrements('id');
            $table->unsignedTinyInteger('type')->index();

            $table->unsignedMediumInteger('category_id')->index(); // product_categories.id

            $table->string('descripcion')->fulltext();
            $table->mediumText('descripcion_completa')->nullable()->index();
            $table->string('no_identificacion', 40)->nullable()->unique();
            $table->string('slug')->nullable()->unique();

            $table->boolean('available_in_pos')->index();
            $table->boolean('available_in_purchases')->index();
            $table->boolean('available_in_ecommerce')->index();
            $table->boolean('available_in_maanufacturing')->index();
            $table->boolean('available_in_quality')->index();
            $table->boolean('available_in_assets')->index();

            $table->string('c_clave_unidad', 3)->nullable()->index(); // sat_clave_unidad.
            $table->unsignedInteger('c_clave_prod_serv')->nullable()->index(); // sat_clave_prod_serv.
            $table->unsignedBigInteger('ean_code')->nullable()->unique();

            $table->decimal('costo', 9, 2)->unsigned()->nullable();
            $table->char('c_moneda', 3)->charset('ascii')->collation('ascii_general_ci')->nullable()->index();
            $table->unsignedTinyInteger('c_objeto_imp')->nullable()->index(); // sat_objeto_imp.
            $table->json('impuestos')->nullable();
            $table->decimal('traslados', 9, 6)->unsigned()->nullable();
            $table->decimal('retenciones', 9, 6)->unsigned()->nullable();

            $table->unsignedTinyInteger('data_lot_enable')->nullable()->index();
            $table->unsignedTinyInteger('data_lot_require')->nullable()->index();
            $table->unsignedTinyInteger('data_series_enable')->nullable()->index();
            $table->unsignedTinyInteger('data_series_require')->nullable()->index();
            $table->unsignedTinyInteger('data_expiration_enable')->nullable()->index();
            $table->unsignedTinyInteger('data_expiration_require')->nullable()->index();
            $table->unsignedTinyInteger('data_warranty_enable')->nullable()->index();
            $table->unsignedMediumInteger('warranty')->nullable()->index();
            $table->unsignedTinyInteger('data_best_before_enable')->nullable()->index();
            $table->unsignedTinyInteger('data_best_before_require')->nullable()->index();
            $table->unsignedTinyInteger('data_observations_enable')->nullable()->index();

            $table->decimal('minimum_unit', 7, 6)->unsigned()->nullable(); // Decimales soportados en la Unidad de Medida
            $table->unsignedTinyInteger('affects_inventory')->nullable()->index();

            $table->unsignedTinyInteger('status')->index();

            // Auditoría
            $table->unsignedMediumInteger('created_by')->nullable()->index(); // users.id

            $table->timestamps();

            // Relaciones
            $table->foreign('category_id')->references('id')->on('product_categories')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign('c_moneda')->references('c_moneda')->on('sat_moneda')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign('c_clave_unidad')->references('c_clave_unidad')->on('sat_clave_unidad')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign('c_clave_prod_serv')->references('c_clave_prod_serv')->on('sat_clave_prod_serv')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign('created_by')->references('id')->on('users')->onUpdate('restrict')->onDelete('restrict');
        });

        Schema::create('product_properties', function (Blueprint $table) {
            $table->smallIncrements('id');

            $table->string('name')->index(); // Ej: "Marca", "Potencia", "Color"
            $table->enum('type', ['text', 'number', 'boolean', 'select'])->default('text');

            $table->timestamps();
        });

        Schema::create('product_store_prices', function (Blueprint $table) {
            $table->mediumIncrements('id');

            $table->unsignedMediumInteger('product_id')->index();
            $table->unsignedSmallInteger('store_id')->index();

            $table->decimal('price', 9, 2)->unsigned()->default(0);
            $table->char('currency', 3)->charset('ascii')->collation('ascii_general_ci');

            $table->boolean('is_discounted')->index();
            $table->decimal('discount_price', 9, 2)->unsigned()->nullable();
            $table->date('discount_start')->nullable();
            $table->date('discount_end')->nullable();

            // Auditoría
            $table->timestamps();

            // Relaciones
            $table->foreign('product_id')->references('id')->on('products')->onUpdate('restrict')->onDelete('cascade');
            $table->foreign('store_id')->references('id')->on('stores')->onUpdate('restrict')->onDelete('cascade');
        });

        Schema::create('product_property', function (Blueprint $table) {
            $table->mediumIncrements('id');

            $table->unsignedMediumInteger('product_id')->index();
            $table->unsignedSmallInteger('property_id')->index();

            $table->string('value')->index(); // Valor de la propiedad, ej: 'Intel', 'DDR4', '500W'

            // Auditoria
            $table->timestamps();

            // Relaciones
            $table->foreign('product_id')->references('id')->on('products')->onUpdate('restrict')->onDelete('cascade');
            $table->foreign('property_id')->references('id')->on('product_properties')->onUpdate('restrict')->onDelete('cascade');
        });

        Schema::create('product_property_values', function (Blueprint $table) {
            $table->mediumIncrements('id');

            $table->unsignedMediumInteger('product_id')->index();
            $table->unsignedSmallInteger('property_id')->index();

            $table->string('value_text')->nullable()->index();
            $table->decimal('value_number', 9, 2)->nullable();
            $table->boolean('value_boolean')->nullable();

            $table->timestamps();

            // Relaciones
            $table->foreign('product_id')->references('id')->on('products')->onUpdate('restrict')->onDelete('cascade');
            $table->foreign('property_id')->references('id')->on('product_properties')->onUpdate('restrict')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('products');
        Schema::dropIfExists('product_properties');
        Schema::dropIfExists('product_store_prices');
        Schema::dropIfExists('product_property');
        Schema::dropIfExists('product_property_values');
    }
};
